"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MAGIC_PROXY_SYMBOL = Symbol('Magic proxy symbol');
exports.MATCHES_INT = /^-?\d+$/;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

function get(input, callback, defaultValue) {
  let currentValue = input;
  const handlers = {
    get(value, key) {
      currentValue = value[key];

      if (currentValue && typeof currentValue === 'object') {
        return new Proxy(currentValue, handlers);
      }

      return new Proxy({}, handlers);
    }

  };
  const proxy = new Proxy(input, handlers);
  callback(proxy);

  if (typeof currentValue === 'undefined' || currentValue === null) {
    if (typeof defaultValue !== 'undefined') {
      return defaultValue;
    }
  }

  return currentValue;
}

exports.get = get;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var get_1 = require("./get");

exports.get = get_1.get;

var set_1 = require("./set");

exports.set = set_1.set;

var update_1 = require("./update");

exports.update = update_1.update;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const utils_1 = require("./utils");

function set(input, callback, newValue) {
  const keys = [];
  const handlers = {
    get(_value, key) {
      keys.push(key);
      return new Proxy({}, handlers);
    }

  };
  const proxy = new Proxy({}, handlers);
  callback(proxy);
  return utils_1.setIn(input, keys, newValue);
}

exports.set = set;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const constants_1 = require("./constants");
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const utils_1 = require("./utils");

function update(input, callback, updater) {
  const keys = [];
  let currentValue = input;
  const handlers = {
    get(value, key) {
      keys.push(key);
      currentValue = value[key];

      if (currentValue && typeof currentValue === 'object') {
        return new Proxy(currentValue, handlers);
      }

      return new Proxy({}, handlers);
    }

  };
  const proxy = new Proxy(input, handlers);
  callback(proxy);
  return utils_1.setIn(input, keys, updater(currentValue));
}

exports.update = update;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const constants_1 = require("./constants");

function setIn(input, keys, newValue) {
  if (!keys.length) {
    return newValue;
  }

  const [firstKey, ...restKeys] = keys;
  const key = constants_1.MATCHES_INT.test(firstKey) ? parseFloat(firstKey) : firstKey;
  let copy;

  if (input === null || typeof input === 'undefined') {
    copy = typeof key === 'number' ? [] : {};
  } else {
    copy = Array.isArray(input) ? [...input] : Object.assign({}, input);
  }

  copy[key] = setIn(copy[key], restKeys, newValue);
  return copy;
}

exports.setIn = setIn;

//# sourceMappingURL=index.js.map